// Copyright Keith D Gregory
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package net.sf.kdgcommons.collections;

import java.util.Iterator;
import java.util.NoSuchElementException;

import junit.framework.TestCase;

public class TestOneElementIterable
extends TestCase
{
    public void testOperation() throws Exception
    {
        OneElementIterable<String> oei = new OneElementIterable<String>("foo");
        Iterator<String> itx = oei.iterator();

        assertTrue(itx.hasNext());
        assertEquals("foo", itx.next());
        assertFalse(itx.hasNext());
    }


    public void testIteratorThrowsOnRemove() throws Exception
    {
        OneElementIterable<String> oei = new OneElementIterable<String>("foo");
        Iterator<String> itx = oei.iterator();
        itx.next();

        try
        {
            itx.remove();
            fail("able to remove from iterator");
        }
        catch (UnsupportedOperationException ex)
        {
            // success
        }
    }


    public void testNextThrowsIfCalledTwice() throws Exception
    {
        OneElementIterable<String> oei = new OneElementIterable<String>("foo");
        Iterator<String> itx = oei.iterator();
        itx.next();

        try
        {
            itx.next();
            fail("able to call next at end of iteration");
        }
        catch (NoSuchElementException ex)
        {
            // success
        }
    }
}
