// Copyright Keith D Gregory
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package net.sf.kdgcommons.lang;


/**
 *  Use this exception as a trap for code paths that you never expect to
 *  be executed (eg, the default clause in an enum-based switch).
 *
 *  @since 1.0.4
 */
public class UnreachableCodeException
extends RuntimeException
{
    private static final long serialVersionUID = 1L;

    public UnreachableCodeException(String message)
    {
        super(message);
    }
}
