// Copyright Keith D Gregory
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package net.sf.kdgcommons.collections;

import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;


public class TestMapBuilder extends TestCase
{
    public void testOperation() throws Exception
    {
        Map<String,String> map = new MapBuilder<String,String>(new HashMap<String,String>())
                                 .put("foo", "bar")
                                 .put("argle", "bargle")
                                 .put("argle", "wargle")  // overwrites
                                 .toMap();

        assertEquals(2, map.size());
        assertEquals("bar", map.get("foo"));
        assertEquals("wargle", map.get("argle"));
    }
}
